package se.plweb.memory.gui;

import java.awt.Color;
import java.awt.Graphics;
import se.plweb.memory.domain.*;

/**
 * 
 * @author Peter Lindblom
 */

public class GameObjectGuiImpl implements GameObjectGui {

	private static final long serialVersionUID = 1L;
    private static final Color ACTIVE_BORDER_COLOR = Color.LIGHT_GRAY;
    private static final Color INACTIVE_BORDER_COLOR = Color.DARK_GRAY;
    private static final Color MATCHED_BORDER_COLOR = Color.RED;
    private static final Color BACKGROUND_COLOR = Color.WHITE;
    private static final Color MATCHED_BACKGROUND_COLOR = Color.LIGHT_GRAY;
    private static final Color MOUSEOVER = Color.BLACK;
	private GameObject gameObject;
	private static final int borderWitdh = 5;
	private static final int borderWitdhX2 = 10;
	
	private int width, height, xTopLeft, yTopLeft;
	public enum GUIState {NORMAL, MOUSE_OVER};
	
	public GameObjectGuiImpl(int value, Position position) {
		gameObject = new GameObjectImpl(value, position);
	}

	public int getValue() {
		return gameObject.getValue();
	}

	public Position getPosition() {
		return gameObject.getPosition();
	}

	public void setPosition(Position position) {
		gameObject.setPosition(position);
	}

	public GameObjectState getState() {
		return gameObject.getState();
	}

	public void setState(GameObjectState value) {
		gameObject.setState(value);
	}

	public void setValue(int value) {
		gameObject.setValue(value);
	}

	@Override
	public String toString() {
        return gameObject.toString(); 		
	}

	public boolean equals(GameObject obj) {
		return gameObject.equals(obj);
	}

    private void paintBackground(Graphics graphics, Color backgroundColor, Color borderColor){
        graphics.setColor(backgroundColor);        
        graphics.fillRect(xTopLeft, yTopLeft, width, height);
        graphics.setColor(borderColor);
        graphics.drawRect(xTopLeft, yTopLeft, width-1, height-1);        
    }

	private void paintValue(Graphics graphics, int value) {
        graphics.setColor(ColorHelper.getColorForValue(value));
        
		if (value % 4 == 0) {
			paintLeftToRight(graphics, value);
		} else if (value % 3 == 0) {
			paintTopToBottom(graphics, value);
		} else if (value % 2 == 0) {
			paintRightToLeft(graphics, value);
		} else {
			paintBottomToTop(graphics, value);
		}
	}

	private void paintLeftToRight(Graphics graphics, int value) {
		
		int tmpValue = getDisplayableValue(value);
		int lineWidth = calculateLineWidth();

		int startX = xTopLeft + borderWitdh;
		int startY = yTopLeft + borderWitdh;		
		int widthExcludingBorders = (width - borderWitdhX2);

		while (tmpValue > 0) {

			if (tmpValue > widthExcludingBorders) {
				graphics.fillRect(startX, startY, widthExcludingBorders, lineWidth);
				tmpValue = tmpValue - widthExcludingBorders;
				startY = startY + lineWidth;
			} else {
				graphics.fillRect(startX, startY, tmpValue, lineWidth);
				tmpValue = 0;
			}
		}
	}
	
	
	private void paintTopToBottom(Graphics graphics, int value) {

		int tmpValue = getDisplayableValue(value);
		int lineWidth = calculateLineWidth();

		int widthExcludingBorders = width - borderWitdhX2;
		int heightExcludingBorders = (height - borderWitdhX2);
		
		int startX = xTopLeft + (widthExcludingBorders - lineWidth);
		int startY = yTopLeft + borderWitdh;

		while (tmpValue > 0) {			
			if (tmpValue > heightExcludingBorders) {
				graphics.fillRect(startX, startY, lineWidth, heightExcludingBorders);
				tmpValue = tmpValue - heightExcludingBorders;
				startX = startX - lineWidth;
			} else {
				graphics.fillRect(startX, startY, lineWidth, tmpValue);
				tmpValue = 0;
			}
		}
	}

	private void paintRightToLeft(Graphics graphics, int value) {
		int tmpValue = getDisplayableValue(value);
		int lineWidth = calculateLineWidth();

		int startX = xTopLeft + borderWitdh;
		int startY = yTopLeft + ((height - borderWitdhX2) - lineWidth);
		int widthExcludingBorders = (width - borderWitdhX2);

		while (tmpValue > 0) {
			if (tmpValue > widthExcludingBorders) {
				graphics.fillRect(startX, startY, widthExcludingBorders, lineWidth);
				tmpValue = tmpValue - widthExcludingBorders;
				startY = startY - lineWidth;
			} else {
				startX = startX + (widthExcludingBorders - tmpValue);
				graphics.fillRect(startX, startY, tmpValue, lineWidth);
				tmpValue = 0;
			}
		}
	}

	private void paintBottomToTop(Graphics graphics, int value) {
		int tmpValue = getDisplayableValue(value);
		int lineWidth = calculateLineWidth();
		
		int startX = xTopLeft + borderWitdh;
		int startY = yTopLeft + borderWitdh;
		
		int heightExcludingBorders = (height - borderWitdhX2);

		while (tmpValue > 0) {
			if (tmpValue > heightExcludingBorders) {
				graphics.fillRect(startX, startY, lineWidth, heightExcludingBorders);
				tmpValue = tmpValue - heightExcludingBorders;
				startX = startX + lineWidth;
			} else {
				startY = startY + (heightExcludingBorders - tmpValue);
				graphics.fillRect(startX, startY, lineWidth, tmpValue);
				tmpValue = 0;
			}
		}
	}

	private int getDisplayableValue(int value) {
		return (value * 10);
	}

	private int calculateLineWidth() {
		Double tmpValue = new Double(Math
				.floor(((height - borderWitdhX2) / 20)));
		return tmpValue.intValue();
	}

	public boolean isInNormalState() {
		return this.gameObject.isInNormalState();
	}

	public boolean hasTheSameValueAndNotTheSameCoordinates(GameObject gameObject) {
		return this.gameObject
				.hasTheSameValueAndNotTheSameCoordinates(gameObject);
	}

	public boolean hasTheSameValueAndTheSameCoordinates(GameObject gameObject) {
		return this.gameObject.hasTheSameValueAndTheSameCoordinates(gameObject);
	}

	public GameObject getClone() {
		GameObject clone = new GameObjectGuiImpl(this.getValue(), this
				.getPosition());
		clone.setState(this.getState());
		return clone;
	}

	public void draw(Graphics graphics, Size size, GUIState guiState) {
		if(size != null){
			this.width = size.getWidth();
			this.height = size.getHeight();
		}
		
		xTopLeft =  this.width * gameObject.getPosition().getXPos(); 
		yTopLeft =  this.height * gameObject.getPosition().getYPos();
		
		switch (gameObject.getState()) {
		case NORMAL_STATE:
			paintBackground(graphics, BACKGROUND_COLOR, ACTIVE_BORDER_COLOR);
			break;
		case DISABLED_STATE:
			paintBackground(graphics, BACKGROUND_COLOR, INACTIVE_BORDER_COLOR);
			break;
		case MATCHED_STATE:                         
			paintBackground(graphics, MATCHED_BACKGROUND_COLOR, MATCHED_BORDER_COLOR);
			paintValue(graphics, gameObject.getValue());
			break;
		case PRESSED_STATE:
			paintBackground(graphics, BACKGROUND_COLOR, INACTIVE_BORDER_COLOR);
			paintValue(graphics, gameObject.getValue());            
			break;
		}
		
		if(guiState.equals(GUIState.MOUSE_OVER)){			
			graphics.setColor(MOUSEOVER);
	        graphics.drawRect((xTopLeft+1), (yTopLeft+1), (width-3), (height-3));
		}
	}

	public boolean isPositionInsideOfGameObject(Position position) {
		
		if(position != null 
			&& position.getXPos() >= xTopLeft 
			&& position.getXPos() < (xTopLeft+width)
			&& position.getYPos() >= yTopLeft
			&& position.getYPos() < (yTopLeft+height)
		){
			return true;
		}
		
		return false;
	}
	
	 
}
